% ------------------------------------------------------------------------
% Simulink model of a three-phase Modular Multilevel Converter
% Copyright : Imperix Switzerland ltd 2021
% Engineers : Nicolas Cherix, Gabriel Fernandez
% -----------------------------------------------------------------------
clc; close all;


%% Bus-side parameters :
Vdc = 800;                      % Bus EMF (V)

%% Three-phase-side parameters :
Vac = 230;
Vac_pk = 325;                   % Nominal peak AC amplitude (source EMF) (V)
fg = 50;                         % Line frequency (Hz)
w = 2*pi*fg; 

%% Submodules parameters :
N = 4;                          % Number of submodules per arm
Csm = 5.0e-3;                   % Submodule capacitance (F)
Rsm = 50e-3;                    % Submodule capacitor ESR (ohm)

%% Branch parameters :
Rb = 60e-3;                     % Parasitic branch resistance (constant part, ohm)
Lb = 2.3e-3;                    % Branch indu ctance (self part, H)

%% Time parameters:
fs = 20e3;                      % Interrupt frequency [Hz]
Ts = 1/fs;                      % Interrupt period [s]
fsw = fs/N;                     % Switching frequency [Hz]
Tsw = 1/fsw;                    % Switching period [s]
Td = 1.5*Ts + 0.5*Tsw;          % Control delay [s]

%% Grid currents regulators
L = Lb/2;
R = Rb/2;
K1 = 1/R;
T1 = L/R;
Tn = T1;
Ti = 2*K1*Td;
Kp_Ig = Tn/Ti;
Ki_Ig = 1/Ti;

%% Partial bus currents regulators
L = 2*Lb;
R = 2*Rb;
K1 = 1/R;
T1 = L/R;
Tn = T1;
Ti = 2*K1*Td;
Kp_Ibus = Tn/Ti;
Ki_Ibus = 1/Ti;

%% Submodules voltage regulators
C = Csm;
T2 = C;
Tdeq = 2*Td;
a = 6;
Tn = a^2*Tdeq;
Ti = a^3*Tdeq^2/T2;
Kp_Vsm  = Tn/Ti;
Ki_Vsm = 1/Ti;

%% Leg voltages regulators
C = Csm/N;
T2 = C;
Tdeq = 2*Td;
a = 6;
Tn = a^2*Tdeq;
Ti = a^3*Tdeq^2/T2;
Kp_Vleg  = Tn/Ti;
Ki_Vleg = 1/Ti;

%% Setvalues :
Pref = 2500;                    % Active power setpoint (W)
Qref = 0000;                    % Reactive power setpoint (Var)
Sref = sqrt(Pref^2 + Qref^2);   % Apparent power setpoint (VA)
phi = atan(Qref/Pref);          % Power factor

Iac_pk_ref = 2/3*Sref/Vac_pk;
Idc_ref = Pref/Vdc;

% %% Initial conditions :
Vsm0 = 1.1*Vdc/N;               % Initial value of the cap voltages _U (V)
Igd_ref0 = Pref/3/Vac*sqrt(2);
Igq_ref0 = Qref/3/Vac*sqrt(2);
Iac0_1 = Iac_pk_ref*sin(phi);
Iac0_2 = Iac_pk_ref*sin(phi-2*pi/3);
Iac0_3 = Iac_pk_ref*sin(phi+2*pi/3);
Idc0 = Idc_ref;
Idc0_1 = Idc0/3;
Idc0_2 = Idc0/3;
Idc0_3 = Idc0/3;

DT = 300e-9;
Vf = 1.2;

